;---------------------------------------------;
;     TENTO PROGRAM TESTUJE RELE-MODUL,       ;
;          PRIPOJENY K CHIPONU2               ;
; simuluje klavesnici a generuje posloupnost: ;
;               "CHIPON2"                     ;
;---------------------------------------------;

	LIST	P = 16F877, R = DEC
	#include	p16f877.inc

	__CONFIG	B'11111100111010'

	#include	flash.inc

;---------------------------------------------
;||||||||||| KONSTANTY PROGRAMU ||||||||||||||
;---------------------------------------------

OFFRAM	EQU	H'20'	; zacatek volne pameti "RAM"


KEYPORT	EQU	PORTC	; port pro klavesnici
KEYMASK	EQU	B'11001111' ; "AND" maska pro praci s portem
#DEFINE	CLOCK	KEYPORT,4	; hodinovy signal
#DEFINE	KEYDATA	KEYPORT,5	; datovy signal
#DEFINE	PREPINAC	KEYPORT,3	; signal ridici rele

	CBLOCK	OFFRAM
	TML:1	; pro utilitu prodlevy
	TMH:1	

	KEYCOUNT:1	; pro generovani "klaves"
	KEYKOD:2
	INDEX:1
	ENDC

	ORG	0
	NOP	; kvuli ZAVADECI
	NOP	; kvuli ZAVADECI
	NOP	; kvuli ZAVADECI

	banksel	OPTION_REG	; nastaveni TMR0 - predelicka: (1:16), zdroj: vnitrni hodiny
	BCF	OPTION_REG,T0CS
	BCF	OPTION_REG,PSA
	BCF	OPTION_REG,2
	BSF	OPTION_REG,1
	BSF	OPTION_REG,0
	banksel	PORTC
	BSF	PREPINAC
	banksel	TRISC
	BCF	PREPINAC	; rizeni rele je aktivni (stav: rozepnuto)
	BSF	CLOCK	; hodiny a data v rezimu vstupu
	BSF	KEYDATA
	banksel	0

	MOVLW	D'255'	; bezpecnostni prodleva
	CALL	DDELAY

	banksel	EEADR	; nastaveni adresy tabulky
	MOVLW	TABULKA / D'256'
	MOVWF	EEADRH
	MOVLW	TABULKA % D'256'
	MOVWF	EEADR
	banksel	0
	MOVLW	D'7'	; bude vyslano 7 klaves
	MOVWF	INDEX
LOOP	CALL	FREAD
	CALL	VYSLIKEY	; prectena a vyslana klavesa (stisknuti)

	MOVLW	D'255'	; zamerne zpozdeni
	CALL	DDELAY

	MOVLW	H'07'	; znak uvolneni klavesy
	MOVWF	KEYKOD+1
	MOVLW	H'E0'
	MOVWF	KEYKOD
	CALL	VYSLIKEY

	MOVLW	D'12'	; drobna prodleva
	CALL	DDELAY

	CALL	EEREGREAD	; opet si precteme cislo klavesy
	CALL	VYSLIKEY	; a vysleme jej na port (uvolneni)

	MOVLW	D'255'	; zamerna prodleva
	CALL	DDELAY

	banksel	EEADR	; inkrementace adresy
	INCF	EEADR,F
	BTFSC	STATUS,Z
	INCF	EEADRH,F
	banksel	0

	DECFSZ	INDEX,F	; jakmile je vsech 7 klaves vyslano,..
	GOTO	LOOP
	BCF	PREPINAC	; ...aktivujeme rele (klavesnice bude pripojena)
KONEC	GOTO	KONEC	; program konci, vse se muze opakovat,...
			; ...pokud mikrokontroler zresetujeme

;------------------------------------------------
; vysle na port kod klavesnice

VYSLIKEY	MOVLW	D'11'	; pocet bitu
	MOVWF	KEYCOUNT
	MOVLW	KEYMASK	; nastavime port pro simulaci...
	ANDWF	KEYPORT,F	; ...otevreneho kolektoru
VYSLILOOP	MOVLW	D'239'	; casovani jednoho obdelniku hodin
	MOVWF	TMR0
	BCF	INTCON,T0IF
	BTFSS	KEYKOD,0
	GOTO	K_ZEMI
	banksel	TRISC	; nastaveni hrany hodin a dat (v rezimu otevreneho kolektoru)
	BSF	KEYDATA
	BCF	CLOCK
	banksel	PORTC
	GOTO	VYSLSKOK_
K_ZEMI	
	banksel	TRISC
	BCF	KEYDATA
	BCF	CLOCK
	banksel	PORTC
VYSLSKOK_	BTFSS	INTCON,T0IF	; cekame na konec casovani obdelniku
	GOTO	$ - 1
	banksel	TRISC	; generujeme opacnou hranu hodin
	BSF	CLOCK
	banksel	PORTC
	MOVLW	D'239'	; nastavine casovani druheho obdelniku
	MOVWF	TMR0
	BCF	INTCON,T0IF
	RRF	KEYKOD+1,F	; posouvame obsah KEYKOD, abychom...
	RRF	KEYKOD,F		; ...meli pripraveny dalsi bit na kraji
	BTFSS	INTCON,T0IF	; cekame na konec casovani obdelniku
	GOTO	$ - 1
	DECFSZ	KEYCOUNT,F	; opakovani cyklu
	GOTO	VYSLILOOP
	banksel	TRISC
	BSF	KEYDATA	; deaktivujeme datovy signal
	banksel	PORTC
	RETURN

FREAD	FREAD_	; cteni z flash
EEREGREAD	
	banksel	EEDATA	; pouhy presun dat mezi registry a pameti
	MOVF	EEDATA,W
	banksel	KEYKOD
	MOVWF	KEYKOD
	banksel	EEDATH
	MOVF	EEDATH,W
	banksel	KEYKOD+1
	MOVWF	KEYKOD+1
	RETURN

DDELAY	MOVWF	TMH	; primitivni zpozdovaci podprogram
	CLRF	TML
	DECFSZ	TML,F
	GOTO	$ - 1
	DECFSZ	TMH,F
	GOTO	$ - 3
	RETURN

;-----------------------------------------
; tabulka posloupnosti klaves

TABULKA	DATA	H'0642'	; C
	DATA	H'0666'	; H
	DATA	H'0486'	; I
	DATA	H'069A'	; P
	DATA	H'0688'	; O
	DATA	H'0462'	; N
	DATA	H'063C'	; 2

	END
